@@ -2,8 +2,8 @@
 #
 # fitstahl.R
 #
-# copyright (c) 2006, Karl W Broman, Johns Hopkins University
-# last modified Aug, 2006
+# copyright (c) 2006-7, Karl W Broman
+# last modified Apr, 2007
 # first written Aug, 2006
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -53,11 +53,15 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
   if(length(class(cross)) < 2 || class(cross)[2] != "cross")
     stop("Input should have class \"cross\".")
 
-  if(class(cross)[1] != "bc")
-    stop("fitstahl only working for backcrosses.")
+  type <- class(cross)[1]
+  if(type != "bc" && type != "f2")
+    stop("fitstahl only working for backcrosses and intercrosses.")
 
   if(!missing(chr)) cross <- subset(cross, chr)
 
+  if(!missing(error.prob) && any(error.prob > 0.5))
+    warning("You probably want to use error.prob <= 0.5.")
+
   if(!missing(m)) { # m was specified
     if(!missing(p)) { # p was specified
 
@@ -77,9 +81,10 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         result <- matrix(ncol=4, nrow=mn)
         colnames(result) <- c("m", "p", "error.prob", "loglik")
         for(i in 1:mn) {
-          result[i,] <- c(m[i], p[i], error.prob[i],
-                          fitstahl.estp.sub(p[i], cross, error.prob[i],
-                                            m[i], tol, maxit))
+          temp <- fitstahl.estp.sub(p[i], cross, error.prob[i],
+                                    m[i], tol, maxit)
+          result[i,] <- c(m[i], p[i], error.prob[i], as.numeric(temp))
+          cross <- replace.map(cross, attr(temp, "map"))
           if(verbose) cat(i,result[i,], "\n")
         }
       }
@@ -100,6 +105,7 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         colnames(result) <- c("m", "p", "error.prob", "loglik")
         for(i in 1:mn) {
           temp <- fitstahl.este(cross, m[i], p[i], tol, maxit)
+          cross <- replace.map(cross, attr(temp, "map"))
           result[i,] <- c(m[i], p[i], temp$est, temp$loglik)
           if(verbose) cat(i, result[i,], "\n")
         }
@@ -122,6 +128,7 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         colnames(result) <- c("m", "p", "error.prob", "loglik")
         for(i in 1:mn) {
           temp <- fitstahl.estp(cross, error.prob[i], m[i], tol, maxit)
+          cross <- replace.map(cross, attr(temp, "map"))
           result[i,] <- c(m[i], temp$est, error.prob[i], temp$loglik)
           if(verbose) cat(i, result[i,], "\n")
         }
@@ -133,6 +140,7 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         colnames(result) <- c("m", "p", "error.prob", "loglik")
         for(i in 1:length(m)) {
           temp <- fitstahl.estpe(cross, m[i], tol, maxit)
+          cross <- replace.map(cross, attr(temp, "map"))
           result[i,] <- c(m[i], temp$est, temp$loglik)
           if(verbose) cat(i, result[i,], "\n")
         }
@@ -160,10 +168,12 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         for(i in 1:mn) {
           # fit the case m=0
           maxll <- fitstahl.estp.sub(p[i], cross, error.prob[i], 0, tol, maxit)
+          cross <- replace.map(cross, attr(maxll, "map"))
           themax <- 0
           if(verbose) cat(i, 0, maxll, "\n")
           for(j in 1:maxm) {
             curll <- fitstahl.estp.sub(p[i], cross, error.prob[i], j, tol, maxit)
+            cross <- replace.map(cross, attr(curll, "map"))
             if(verbose) cat(i, j, curll, "\n")
             if(curll < maxll) break
             if(curll > maxll) {
@@ -185,14 +195,16 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         for(i in 1:mn) {
           # fit the case m=0
           temp <- fitstahl.este(cross, 0, p[i], tol, maxit)
+          cross <- replace.map(cross, attr(temp, "map"))
           maxll <- temp$loglik
           themax <- 0
           themaxe <- temp$est
-          if(verbose) cat(i, 0, maxll, "\n")
+          if(verbose) cat(i, 0, 0, temp$est, maxll, "\n")
           for(j in 1:maxm) {
             temp <- fitstahl.este(cross, j, p[i], tol, maxit)
+            cross <- replace.map(cross, attr(temp, "map"))
             curll <- temp$loglik
-            if(verbose) cat(i, j, curll, "\n")
+            if(verbose) cat(i, j, p[i], temp$est, curll, "\n")
             if(curll < maxll) break
             if(curll > maxll) {
               maxll <- curll
@@ -214,13 +226,15 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         for(i in 1:mn) {
           # fit the case m=0 (in which case p doesn't matter)
           maxll <- fitstahl.estp.sub(0, cross, error.prob[i], 0, tol, maxit)
+          cross <- replace.map(cross, attr(maxll, "map"))
           themax <- 0
           themaxp <- 0
-          if(verbose) cat(i, 0, maxll, "\n")
+          if(verbose) cat(i, 0, 0, error.prob[i], maxll, "\n")
           for(j in 1:maxm) {
             temp <- fitstahl.estp(cross, error.prob[i], j, tol, maxit)
+            cross <- replace.map(cross, attr(temp, "map"))
             curll <- temp$loglik
-            if(verbose) cat(i, j, curll, "\n")
+            if(verbose) cat(i, j, temp$est, error.prob[i], curll, "\n")
             if(curll < maxll) break
             if(curll > maxll) {
               maxll <- curll
@@ -239,14 +253,16 @@ function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         colnames(result) <- c("m", "p", "error.prob", "loglik")
         # fit the case m=0 (in which case p doesn't matter)
         temp <- fitstahl.este(cross, 0, 0, tol, maxit)
+        cross <- replace.map(cross, attr(temp, "map"))
         maxll <- temp$loglik
         themax <- 0
         themaxest <- c(0, temp$est) 
-        if(verbose) cat(i, 0, maxll, "\n")
+        if(verbose) cat(0, 0, temp$est, maxll, "\n")
         for(j in 1:maxm) {
           temp <- fitstahl.estpe(cross, j, tol, maxit)
+          cross <- replace.map(cross, attr(temp, "map"))
           curll <- temp$loglik
-          if(verbose) cat(i, j, curll, "\n")
+          if(verbose) cat(j, temp$est, curll, "\n")
           if(curll < maxll) break
           if(curll > maxll) {
             maxll <- curll
@@ -276,21 +292,32 @@ function(cross, error.prob=0.0001, m=0, tol=1e-4, maxit=4000)
 
   # make sure we fit p=0
   temp <- fitstahl.estp.sub(0, cross, error.prob, m, tol, maxit)
+
   if(temp >= out[[2]]) {
     est <- 0
     loglik <- temp
+    themap <- attr(temp, "map")
   }
   else {
     est <- out[[1]]
     loglik <- out[[2]]
+    themap <- attr(out[[2]], "map")
   }
-  list(est=est, loglik=loglik)
+  out <- list(est=est, loglik=loglik)
+  attr(out, "map") <- themap
+  out
 }
 
 fitstahl.estp.sub <-
 function(p, cross, error.prob=0.0001, m=0, thetol=1e-4, maxit=4000)
-  sum(sapply(est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
-                     maxit=maxit), function(a) attr(a, "loglik")))
+{  
+  newmap <- est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
+                    maxit=maxit)
+  out <- sum(sapply(newmap, function(a) attr(a, "loglik")))
+
+  attr(out, "map") <- newmap
+  out
+}
 
 ######################################################################
 # fitstahl.este: estimate error.prob for fixed m and p
@@ -298,7 +325,7 @@ function(p, cross, error.prob=0.0001, m=0, thetol=1e-4, maxit=4000)
 fitstahl.este <-
 function(cross, m=0, p=0, tol=1e-4, maxit=4000)
 {
-  out <- optimize(fitstahl.este.sub, interval=c(0,1), maximum=TRUE,
+  out <- optimize(fitstahl.este.sub, interval=c(0,0.5), maximum=TRUE,
                   cross=cross, m=m, p=p, thetol=tol, maxit=maxit)
 
   # make sure we fit error.prob=0
@@ -306,19 +333,28 @@ function(cross, m=0, p=0, tol=1e-4, maxit=4000)
   if(temp >= out[[2]]) {
     est <- 0
     loglik <- temp
+    themap <- attr(temp, "map")
   }
   else {
     est <- out[[1]]
     loglik <- out[[2]]
+    themap <- attr(out[[2]], "map")
   }
 
-  list(est=est, loglik=loglik)
+  out <- list(est=est, loglik=loglik)
+  attr(out, "map") <- themap
+  out
 }
 
 fitstahl.este.sub <-
-function(error.prob, cross, m=0, p=0, thetol=1e-4, maxit=4000)
-  sum(sapply(est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
-                     maxit=maxit), function(a) attr(a, "loglik")))
+function(error.prob, cross, m=0, p=0, thetol=1e-4, maxit=4000) 
+{  
+  newmap <- est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
+                    maxit=maxit)
+  out <- sum(sapply(newmap, function(a) attr(a, "loglik")))
+  attr(out, "map") <- newmap
+  out
+}
 
 
 ######################################################################
@@ -327,34 +363,48 @@ function(error.prob, cross, m=0, p=0, thetol=1e-4, maxit=4000)
 fitstahl.estpe <-
 function(cross, m=0, tol=1e-4, maxit=4000)
 {
+  if(m==0) { # don't need to estimate p
+    out <- fitstahl.este(cross, 0, 0, tol, maxit)
+    out$est <- c(0, out$est)
+    return(out)
+  }
+
   out <- optim(c(0.1,0.01), fitstahl.estpe.sub, method="L-BFGS-B",
-               lower=c(0,0), upper=c(1,1),
-               control=list(fnscale=-1, maxit=maxit, reltol=tol),
+               lower=c(0,0), upper=c(1,0.5),
+               control=list(fnscale=-1, maxit=maxit, factr=tol/1e-15),
                cross=cross, m=m, thetol=tol, maxit=maxit)
 
   if(out$convergence !=0) 
     warning(" Didn't converge.")
-  print(out$counts)
 
   # make sure we fit the case p=0
-  temp <- fitstahl.estp(cross, m, 0, tol, maxit)
-  templl <- attr(temp, "loglik")
+  temp <- fitstahl.este(cross, m, 0, tol, maxit)
+  templl <- temp$loglik
   if(templl >= out$value) {
-    est <- c(0, temp) 
+    est <- c(0, temp$est) 
     loglik <- templl
+    themap <- attr(temp, "map")
   }
   else {
     est <- out$par
     loglik <- out$value
+    themap <- est.map(x, error.prob=out$par[2], m=m, p=out$par[1],
+                      maxit=maxit, tol=tol)
   }
   names(est) <- c("p", "error.prob")
-  list(est=est, loglik=loglik)
+  out <- list(est=est, loglik=loglik)
+  attr(out, "map") <- themap
+  out
 }
 
 fitstahl.estpe.sub <-
 function(x, cross, m=0, thetol=1e-4, maxit=4000)
-  sum(sapply(est.map(cross, error.prob=x[2], m=m, p=x[1], tol=thetol,
-                     maxit=maxit), function(a) attr(a, "loglik")))
-
+{
+  newmap <- est.map(cross, error.prob=x[2], m=m, p=x[1], tol=thetol,
+                    maxit=maxit)
+  out <- sum(sapply(newmap, function(a) attr(a, "loglik")))
+  attr(out, "map") <- newmap
+  out
+}
 
 # end of fitstahl.R
